import urllib
import json
import settings

import exceptions
# import pdb

# see: http://en.wikipedia.org/wiki/65536_(number)
# api example: https://qrng.anu.edu.au/API/api-demo.php


# todo: should error if result is bad
def request_qnum_set(length=1):
    result = None
    url = settings.ANU_API_URL + urllib.urlencode({
        'type': settings.QNUM_TYPE,
        'size': settings.QNUM_SIZE,
        'length': length,
    })

    response = json.loads(urllib.urlopen(url).read())

    if(
        response['success'] is True
    ):
        result = response
    else:
        raise exceptions.AnuApiFailed(length)

    return result


def qnum_set_to_ranged_set(qset, max, min=1):
    # qnums range from 0 to our MAX_INT
    # factor by which to divide our resulting qnum
    # qnum / modulus will reduce the original numbe (0 - MAX_INT)
    #   to our range (0 - max)
    modulus = settings.MAX_INT / max

    # + min, typically 1 offsets our values by one because most d & d die
    #   do not have a zero value
    ranged_set = [(x / modulus) + min for x in qset]
    return ranged_set


def qnum_ranged_set(length, max, min=1):
    qset_response = request_qnum_set(length)
    qset = qset_response['data']
    ranged_set = qnum_set_to_ranged_set(qset, max, min)
    return ranged_set




"""Usage:
    pydie roll <multiplier> <die> [<modifier>]
    pydie [-h | --help]

    Generate a random n-sided dice role.

    Options:
        -h --help  print help information for this program

"""

import sys
from pydie import roll
from docopt import docopt


def main():
    arguments = docopt(__doc__)

    if(arguments['roll']):
        multiplier = int(arguments['<multiplier>'])
        die = arguments['<die>']
        mod_str = arguments['<modifier>']

        print '\nRolling {0} {1}s'.format(arguments['<multiplier>'], arguments['<die>'])

        result = roll(
            multiplier,
            die,
            mod_str
        )

    if(result['success']):
        print """\nYour result is {0}!
Your original rolls were {1}
Your original total was {2}
You applied the following modifiers {3}
""".format(result['modified_result'],
                    result['roll_results'],
                    result['original_result'],
                    result['modifier_str']
            )


if __name__ == "__main__":
    sys.exit(main())



class PyDieError(Exception):
    pass


class AnuApiFailed(PyDieError):
    def __init__(self, length):
        self.length = length

    def __str__(self):
        return "The attempt to access the ANU Server API has failed. The max length allowed is {0} but the max length spcified was {1}".format(self.length)



import re
import qrand

# import pdb


def roll(multiplier, die, modifier_str=''):
    modifier_str = '' if modifier_str is None else modifier_str

    # re will capture the digits from "d20"
    rDie = r'(\d+)'

    # re will match a +n or -n and capture the "n"
    rBonuses = r'\+(\d+)'
    rPenalties = r'-(\d+)'

    result = {
        'multiplier': multiplier,
        'die': die,
        'modifier_str': modifier_str,
        'roll_results': [],
        'original_result': -1,
        'modified_result': -1,
        'bonuses': [],
        'penalties': [],
        'success': False
    }

    # number proceeding the d is our number of sides or "max_num"
    max_num = int(re.findall(rDie, die)[0])

    # get random number set from qrand
    roll_results = qrand.qnum_ranged_set(multiplier, max_num)

    # store original result
    original_result = sum(roll_results)

    # apply modifiers to result and store modified result
    # list comp is, for all captured matches in modified_str
    #   convert to int and sum
    modified_result = original_result
    bonuses = re.findall(rBonuses, modifier_str)
    penalties = re.findall(rPenalties, modifier_str)
    modified_result += sum([int(b) for b in bonuses])
    modified_result -= sum([int(p) for p in penalties])

    # update result
    result['roll_results'] = roll_results
    result['original_result'] = original_result
    result['bonuses'] = [int(b) for b in bonuses]
    result['penalties'] = [int(p) for p in penalties]
    result['modified_result'] = modified_result

    if original_result != -1:
        result["success"] = True

    return result



#!/usr/bin/env python

import os
import sys
from setuptools import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

os.environ['PYTHONDONTWRITEBYTECODE'] = '1'

packages = [
    'pydie'
]

package_data = {'pydie': ['LICENSE.md', 'README.md']}

requires = [
    'setuptools',
    'docopt==0.6.1'
]

entry_points = {
    'console_scripts': ['pydie = pydie.cli:main']
}

setup(
    url='http://github.com/aubricus/pydie',
    name='pydie',
    version='0.1.7',
    description='Generate a random roll from a n-number of an n-sided die',
    author='Aubrey Taylor',
    author_email='aubricus@gmail.com',
    packages=packages,
    package_data=package_data,
    install_requires=requires,
    entry_points=entry_points,
    package_dir={'pydie': 'pydie'},
    long_description=open('README.rst').read(),
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),

)

del os.environ['PYTHONDONTWRITEBYTECODE']
